import { world, system } from "@minecraft/server";
import { hammerList } from "./tools/hammerList";
import { findAndBreakNearBlocks } from "./tools/hammerEvents";
import { decreaseDurability } from "./itemEvents";

world.beforeEvents.playerBreakBlock.subscribe((event) => {
    const block = event.block;
    const player = event.player;
    const itemUsed = event.itemStack;
    
    if (hammerList[itemUsed?.typeId]) {
        const radius = 
            hammerList[itemUsed?.typeId].breakArea.radius;
        const depth = hammerList[itemUsed?.typeId].breakArea.depth;

        system.run(() => {
            findAndBreakNearBlocks(player, block.location, itemUsed, radius, depth);
        });
    }
});

world.beforeEvents.worldInitialize.subscribe((eventData) => {
    eventData.itemComponentRegistry.registerCustomComponent("justhammers:hammer_durability", {

        onMineBlock: (event) => {
            const player = event.source;
            const itemUsed = event.itemStack;

            if (hammerList[itemUsed?.typeId]) {
                const damageAmount = hammerList[itemUsed.typeId].breakDamage;
                if (damageAmount > 0) {
                    decreaseDurability(player, itemUsed, damageAmount);
                }
            }
        },

        onBeforeDurabilityDamage: (event) => {
            event.durabilityDamage = 0;
        },
    });
});