import { world } from "@minecraft/server";
import { dropItem } from "../itemEvents";
import { hammerList } from "./hammerList";
import { ignoredBlocks, mineralBlocks, silkTouchBlocks } from "../blockList";

export function findAndBreakNearBlocks(player, blockPos, item, radius, depth) {
    const playerDirection = getCardinalDirection(player);
    const area = getAreaOfEffect(playerDirection, blockPos, radius, depth);
        
    if (!area || !item) return;

    for (let x = area.min.x; x <= area.max.x; x++) {
        for (let y = area.min.y; y <= area.max.y; y++) {
            for (let z = area.min.z; z <= area.max.z; z++) {
                const block = player.dimension.getBlock({x, y, z});
                if (!ignoredBlocks.includes(block?.typeId)) {
                    const location = block.location;
                    const minableOres = hammerList[item?.typeId].tier;
                    const fortuneLevel = item.getComponent('enchantable')?.getEnchantment('fortune')?.level;
                    const silkTouchLevel = item.getComponent('enchantable')?.getEnchantment('silk_touch')?.level;

                    if (world.gameRules.doTileDrops == true && player?.getGameMode() !== "creative") {
                        if (mineralBlocks[block?.typeId] && !silkTouchLevel) {
                            const mineralDrop = mineralBlocks[block?.typeId].drop;

                            if (minableOres.includes(block?.typeId))  {
                                if (fortuneLevel > 0) {
                                    const extraAmount = fortuneAmount(fortuneLevel);

                                    dropItem(block, mineralDrop, extraAmount);
                                }
                                player.runCommand(`setblock ${location.x} ${location.y} ${location.z} air destroy`);
                            }
                        } else if (silkTouchBlocks.includes(block?.typeId) && silkTouchLevel) {
                            if (mineralBlocks[block?.typeId] && minableOres.includes(block?.typeId) || !mineralBlocks[block?.typeId]) {
                                dropItem(block, block?.typeId, 1);
                                block.setType("minecraft:air");
                            }
                        } else {
                            player.runCommand(`setblock ${location.x} ${location.y} ${location.z} air destroy`);
                        }
                    } else {
                        block.setType("minecraft:air")
                    }
                }
            }
        }
    }
};

function getCardinalDirection(player) {
    if (player !== undefined) {
        const { x, y, z } = player.getViewDirection();
        if (y > 0.5) {
            return "Up"
        } else if (y < -0.5) {
            return "Down"
        } else if (Math.abs(x) > Math.abs(z)) {
            return x > 0 ? "East" : "West";
        } else {
            return z > 0 ? "South" : "North";
        }
    } else {
        return undefined
    }
};

function getAreaOfEffect(direction, blockPos, radius, depth) {
    const size = Math.floor(radius / 2);
    const offset = size - 1;

    switch (direction) {
        case "Up":
        case "Down":
            return {
                min: {
                    x: blockPos.x - size,
                    y: direction === "Down" ? blockPos.y - depth + 1 : blockPos.y,
                    z: blockPos.z - size
                },
                max: {
                    x: blockPos.x + size,
                    y: direction === "Up" ? blockPos.y + depth - 1 : blockPos.y,
                    z: blockPos.z + size
                }
            };
        case "North":
            return {
                min: {
                    x: blockPos.x - size,
                    y: blockPos.y - size + offset,
                    z: blockPos.z - depth + 1
                },
                max: {
                    x: blockPos.x + size,
                    y: blockPos.y + size + offset,
                    z: blockPos.z
                }
            };
        case "South":
            return {
                min: {
                    x: blockPos.x - size,
                    y: blockPos.y - size + offset,
                    z: blockPos.z
                },
                max: {
                    x: blockPos.x + size,
                    y: blockPos.y + size + offset,
                    z: blockPos.z + depth - 1
                }
            };
        case "East":
            return {
                min: {
                    x: blockPos.x,
                    y: blockPos.y - size + offset,
                    z: blockPos.z - size
                },
                max: {
                    x: blockPos.x + depth - 1,
                    y: blockPos.y + size + offset,
                    z: blockPos.z + size
                }
            };
        case "West":
            return {
                min: {
                    x: blockPos.x - depth + 1,
                    y: blockPos.y - size + offset,
                    z: blockPos.z - size
                },
                max: {
                    x: blockPos.x,
                    y: blockPos.y + size + offset,
                    z: blockPos.z + size
                }
            };
        default:
            return undefined;
    }
};

function fortuneAmount(level) {
    const chance = 0.33;
    let amountToDrop = 1;

    for (let i = 0; i < level; i++) {
        if (Math.random() < 0.33) {
            amountToDrop++
        }
    }
    return amountToDrop;
};